/***************************************************************************
 * This file is part of the project AqFoxExt.                              *
 * AqFoxExt (c) by 2011 Martin Preuss, all rights reserved.                *
 *                                                                         *
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fxx_app.hpp"
#include "fxx_icondict.hpp"

#include <gwenhywfar/gwenhywfar.h>
#include <gwenhywfar/pathmanager.h>
#include <gwenhywfar/debug.h>



static const unsigned char _iconNotFound16x16[]={
  0x89,0x50,0x4e,0x47,0x0d,0x0a,0x1a,0x0a,0x00,0x00,0x00,0x0d,0x49,0x48,0x44,0x52,
  0x00,0x00,0x00,0x10,0x00,0x00,0x00,0x10,0x08,0x06,0x00,0x00,0x00,0x1f,0xf3,0xff,
  0x61,0x00,0x00,0x00,0x09,0x70,0x48,0x59,0x73,0x00,0x00,0x0d,0x61,0x00,0x00,0x0c,
  0xc4,0x01,0xee,0x96,0x40,0xb5,0x00,0x00,0x00,0x12,0x49,0x44,0x41,0x54,0x38,0x8d,
  0x63,0x60,0x18,0x05,0xa3,0x60,0x14,0x8c,0x02,0x08,0x00,0x00,0x04,0x10,0x00,0x01,
  0x7f,0x4d,0x78,0x8e,0x00,0x00,0x00,0x00,0x49,0x45,0x4e,0x44,0xae,0x42,0x60,0x82
};




FXXApp::FXXApp(const FXString& name, const FXString& vendor)
:FXXTrayApp(name, vendor)
,m_sizeSmallIcons(FXSize(16, 16))
,m_sizeMediumIcons(FXSize(32, 32))
,m_sizeBigIcons(FXSize(24, 24))
,m_markColor1(FXRGB(255, 244, 164))
,m_restartRequested(false)
,m_iconSource(NULL)
,m_iconDict(NULL)
,m_emptyIcon(NULL)
{
  m_iconSource=new FXIconSource(this);
  m_iconDict=new FXXIconDict(this);
}



FXXApp::~FXXApp() {
  if (m_iconSource)
    delete m_iconSource;
  if (m_iconDict)
    delete m_iconDict;
  if (m_emptyIcon)
    delete m_emptyIcon;
}



void FXXApp::init(int& argc, char** argv, bool connect) {
  int rv;

  FXXTrayApp::init(argc, argv, connect);

  rv=GWEN_Init();
  if (rv<0) {
    DBG_ERROR(0, "Unable to init GWEN: %d", rv);
    return;
  }

  /* define ressource path */
  rv=GWEN_PathManager_DefinePath(FXX_PM_LIBNAME, FXX_PM_RESSOURCEDIR);
  if (rv<0) {
    DBG_ERROR(0, "Unable to define path %s/%s: %d", FXX_PM_LIBNAME, FXX_PM_RESSOURCEDIR, rv);
    return;
  }

#if defined(OS_WIN32) || defined(ENABLE_LOCAL_INSTALL)
  /* add folder relative to EXE */
  GWEN_PathManager_AddRelPath(FXX_PM_LIBNAME,
                              FXX_PM_LIBNAME,
                              FXX_PM_RESSOURCEDIR,
                              AQFOXEXT_DATA_DIR,
                              GWEN_PathManager_RelModeExe);
#else
  /* add absolute folder */
  GWEN_PathManager_AddPath(FXX_PM_LIBNAME,
                           FXX_PM_LIBNAME,
                           FXX_PM_RESSOURCEDIR,
                           AQFOXEXT_DATA_DIR);
#endif
}



void FXXApp::fini() {
  int rv;

  /* unddefine ressource path */
  rv=GWEN_PathManager_UndefinePath(FXX_PM_LIBNAME, FXX_PM_RESSOURCEDIR);
  if (rv<0) {
    DBG_ERROR(0, "Unable to undefine path %s/%s: %d", FXX_PM_LIBNAME, FXX_PM_RESSOURCEDIR, rv);
    GWEN_Fini();
    return;
  }

  rv=GWEN_Fini();
  if (rv<0) {
    DBG_ERROR(0, "Unable to deinit GWEN: %d", rv);
    return;
  }
}



GWEN_DB_NODE *FXXApp::getDialogConfig(const char *name) {
  GWEN_CONFIGMGR *cm;

  cm=getConfigManager();
  if (cm) {
    int rv;
    GWEN_DB_NODE *db;

    rv=GWEN_ConfigMgr_GetGroup(cm, "dialogs", name, &db);
    if (rv<0) {
      DBG_ERROR(0, "Error getting dialog configuration group (%d)", rv);
      return NULL;
    }

    return db;
  }
  else {
    DBG_ERROR(0, "No config manager");
    return NULL;
  }
}



int FXXApp::setDialogConfig(const char *name, GWEN_DB_NODE *db) {
  GWEN_CONFIGMGR *cm;

  cm=getConfigManager();
  if (cm) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(cm, "dialogs", name);
    if (rv<0) {
      DBG_ERROR(0, "Unable to lock configuration group (%d)", rv);
      return rv;
    }
    else {
      rv=GWEN_ConfigMgr_SetGroup(cm, "dialogs", name, db);
      if (rv<0) {
	DBG_ERROR(0, "Error setting configuration group (%d)", rv);
	GWEN_ConfigMgr_UnlockGroup(cm, "dialogs", name);
	return rv;
      }

      rv=GWEN_ConfigMgr_UnlockGroup(cm, "dialogs", name);
      if (rv<0) {
	DBG_ERROR(0, "Unable to lock configuration group (%d)", rv);
	return rv;
      }
    }

    return 0;
  }
  else {
    DBG_ERROR(0, "No config manager");
    return GWEN_ERROR_GENERIC;
  }
}



FXString FXXApp::getIconFilePath(const char *name) {
  GWEN_BUFFER *fbuf;
  int rv;
  FXString tIcon;
  FXString t;

  fbuf=GWEN_Buffer_new(0, 256, 0, 1);

  /* direct approach */
  tIcon=FXString("icons" GWEN_DIR_SEPARATOR_S)+FXString(name);
  rv=GWEN_PathManager_FindFile(FXX_PM_LIBNAME, FXX_PM_RESSOURCEDIR, tIcon.text(), fbuf);
  if (rv==0) {
    FXString str;

    str=FXString(GWEN_Buffer_GetStart(fbuf));
    GWEN_Buffer_free(fbuf);
    return str;
  }

  /* find as png */
  t=tIcon+".png";
  rv=GWEN_PathManager_FindFile(FXX_PM_LIBNAME, FXX_PM_RESSOURCEDIR, t.text(), fbuf);
  if (rv==0) {
    FXString str;

    str=FXString(GWEN_Buffer_GetStart(fbuf));
    GWEN_Buffer_free(fbuf);
    return str;
  }

  /* find as jpg */
  t=tIcon+".jpg";
  rv=GWEN_PathManager_FindFile(FXX_PM_LIBNAME, FXX_PM_RESSOURCEDIR, t.text(), fbuf);
  if (rv==0) {
    FXString str;

    str=FXString(GWEN_Buffer_GetStart(fbuf));
    GWEN_Buffer_free(fbuf);
    return str;
  }
  GWEN_Buffer_free(fbuf);

  return "";
}



FXIcon *FXXApp::loadIconFile(const char *name) {
  FXIcon *ic;

  ic=m_iconSource->loadIconFile(name);
  if (ic==NULL) {
    DBG_ERROR(0, "Unable to load icon file \"%s\"", name);
    return NULL;
  }
  return ic;
}



FXIcon *FXXApp::loadIcon(const char *name) {
  FXIcon *ic;
  FXString fpath;
  bool wantFaded=false;

  if (*name=='!') {
    wantFaded=true;
    name++;
  }

  fpath=getIconFilePath(name);
  if (fpath.empty()) {
    DBG_ERROR(0, "Icon \"%s\" not found", name);
    return NULL;
  }
  else {
    DBG_ERROR(0, "Path for icon \%s\" is \"%s\"", name, fpath.text());
    ic=m_iconSource->loadIconFile(fpath.text());
    if (ic==NULL) {
      DBG_ERROR(0, "Unable to load icon file \"%s\"", fpath.text());
      return NULL;
    }
    if (wantFaded)
      /* return faded version */
      ic->fade(FXRGBA(255, 255, 255, 255), 32);
  }

  return ic;
}



FXIcon *FXXApp::getEmptyIcon() {
  if (m_emptyIcon==NULL)
    m_emptyIcon=m_iconSource->loadIconData(_iconNotFound16x16, "png");

  return m_emptyIcon;
}



FXIcon *FXXApp::getIcon(const char *name) {
  FXIcon *ic;

  ic=m_iconDict->getIcon(name);
  if (ic==NULL) {
    DBG_ERROR(0, "Icon \"%s\" not found", name);
    ic=getEmptyIcon();
  }

  return ic;
}







